"""Pin definitions for Clara AGX Xavier."""

from adafruit_blinka.microcontroller.tegra.t194 import pin

SDA = pin.SDA
SCL = pin.SCL
SDA_1 = pin.SDA_1
SCL_1 = pin.SCL_1

D4 = pin.Q06
D5 = pin.AA03
D6 = pin.AA02
D7 = pin.Z07
D8 = pin.Z06
D9 = pin.Z04
D10 = pin.Z05
D11 = pin.Z03
D12 = pin.BB01
D13 = pin.AA00
D16 = pin.R05
D17 = pin.R04
D18 = pin.H07
D19 = pin.I02
D20 = pin.I01
D21 = pin.I00
D22 = pin.N01
D23 = pin.BB00
D24 = pin.H00
D25 = pin.Q01
D26 = pin.AA01
D27 = pin.P04

CE1 = D7
CE0 = D8
MISO = D9
MOSI = D10
SCLK = D11
SCK = D11



"""Platform agnostic time implementation"""

from adafruit_blinka import agnostic

# We intentionally are patching into this namespace so skip the wildcard check.
# pylint: disable=unused-wildcard-import,wildcard-import

if agnostic.implementation == "circuitpython":
    from time import *
elif agnostic.implementation == "micropython":
    import utime
    from utime import sleep

    from ucollections import namedtuple

    _struct_time = namedtuple(
        "struct_time",
        (
            "tm_year",
            "tm_mon",
            "tm_mday",
            "tm_hour",
            "tm_min",
            "tm_sec",
            "tm_wday",
            "tm_yday",
            "tm_isdst",
        ),
    )

    # pylint: disable=too-many-arguments
    def _marshal_time(
        tm_year,
        tm_mon,
        tm_mday,
        tm_hour=0,
        tm_min=0,
        tm_sec=0,
        tm_wday=-1,
        tm_yday=-1,
        tm_isdst=-1,
    ):
        """Construct struct_time with default values."""
        _struct_time(
            tm_year,
            tm_mon,
            tm_mday,
            tm_hour,
            tm_min,
            tm_sec,
            tm_wday,
            tm_yday,
            tm_isdst,
        )

    def struct_time(time_tuple):
        """Create a struct_time"""
        return _marshal_time(*time_tuple)

    # pylint: disable=invalid-name
    _total_ms = 0
    _prev_ticks_ms = utime.ticks_ms()

    def monotonic():
        """A monotonically increasing time in seconds. No defined start time."""
        # Assumes that monotonic is called more frequently than the wraparound of micropython's
        # utime.ticks_ms()
        global _prev_ticks_ms, _total_ms  # pylint: disable=global-statement
        ticks_ms = utime.ticks_ms()
        _total_ms += utime.ticks_diff(ticks_ms, _prev_ticks_ms)
        _prev_ticks_ms = ticks_ms
        return _total_ms * 0.001



import time
import board
import digitalio

print("hello blinka!")

led = digitalio.DigitalInOut(board.D4)
led.direction = digitalio.Direction.OUTPUT

button = digitalio.DigitalInOut(board.D18)
button.direction = digitalio.Direction.INPUT
button.pull = digitalio.Pull.DOWN

while True:
    led.value = button.value
    time.sleep(0.1)



"""
`adafruit_blinka` - Runtime utility objects for re-implementation of CircuitPython API
======================================================================================

* Author(s): cefn
"""


class Enum:
    """
    Object supporting CircuitPython-style of static symbols
    as seen with Direction.OUTPUT, Pull.UP
    """

    def __repr__(self):
        """
        Assumes instance will be found as attribute of own class.
        Returns dot-subscripted path to instance
        (assuming absolute import of containing package)
        """
        cls = type(self)
        for key in dir(cls):
            if getattr(cls, key) is self:
                return "{}.{}.{}".format(cls.__module__, cls.__qualname__, key)
        return repr(self)

    @classmethod
    def iteritems(cls):
        """
        Inspects attributes of the class for instances of the class
        and returns as key,value pairs mirroring dict#iteritems
        """
        for key in dir(cls):
            val = getattr(cls, key)
            if isinstance(cls, val):
                yield (key, val)


class ContextManaged:
    """An object that automatically deinitializes hardware with a context manager."""

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.deinit()

    # pylint: disable=no-self-use
    def deinit(self):
        """Free any hardware used by the object."""
        return

    # pylint: enable=no-self-use


class Lockable(ContextManaged):
    """An object that must be locked to prevent collisions on a microcontroller resource."""

    _locked = False

    def try_lock(self):
        """Attempt to grab the lock. Return True on success, False if the lock is already taken."""
        if self._locked:
            return False
        self._locked = True
        return True

    def unlock(self):
        """Release the lock so others may use the resource."""
        if self._locked:
            self._locked = False
        else:
            raise ValueError("Not locked")


def patch_system():
    """Patch modules that may be different due to the platform."""
    # pylint: disable=import-outside-toplevel
    import sys
    from adafruit_blinka.agnostic import time

    # pylint: enable=import-outside-toplevel

    sys.modules["time"] = time



"""FT232H pin names"""


class Pin:
    """A basic Pin class for use with FT232H."""

    IN = 0
    OUT = 1
    LOW = 0
    HIGH = 1

    ft232h_gpio = None

    def __init__(self, pin_id=None):
        # setup GPIO controller if not done yet
        # use one provided by I2C as default
        if not Pin.ft232h_gpio:
            # pylint: disable=import-outside-toplevel
            from pyftdi.i2c import I2cController

            # pylint: enable=import-outside-toplevel

            i2c = I2cController()
            i2c.configure("ftdi://ftdi:ft232h/1")
            Pin.ft232h_gpio = i2c.get_gpio()
        # check if pin is valid
        if pin_id:
            if Pin.ft232h_gpio.all_pins & 1 << pin_id == 0:
                raise ValueError("Can not use pin {} as GPIO.".format(pin_id))
        # ID is just bit position
        self.id = pin_id

    def init(self, mode=IN, pull=None):
        """Initialize the Pin"""
        if not self.id:
            raise RuntimeError("Can not init a None type pin.")
        # FT232H does't have configurable internal pulls?
        if pull:
            raise ValueError("Internal pull up/down not currently supported.")
        pin_mask = Pin.ft232h_gpio.pins | 1 << self.id
        current = Pin.ft232h_gpio.direction
        if mode == self.OUT:
            current |= 1 << self.id
        else:
            current &= ~(1 << self.id)
        Pin.ft232h_gpio.set_direction(pin_mask, current)

    def value(self, val=None):
        """Set or return the Pin Value"""
        if not self.id:
            raise RuntimeError("Can not access a None type pin.")
        current = Pin.ft232h_gpio.read(with_output=True)
        # read
        if val is None:
            return 1 if current & 1 << self.id != 0 else 0
        # write
        if val in (self.LOW, self.HIGH):
            if val == self.HIGH:
                current |= 1 << self.id
            else:
                current &= ~(1 << self.id)
            # must mask out any input pins
            Pin.ft232h_gpio.write(current & Pin.ft232h_gpio.direction)
            return None
        # release the kraken
        raise RuntimeError("Invalid value for pin")


# create pin instances for each pin
# D0 to D3 are used by I2C/SPI
D4 = Pin(4)
D5 = Pin(5)
D6 = Pin(6)
D7 = Pin(7)
C0 = Pin(8)
C1 = Pin(9)
C2 = Pin(10)
C3 = Pin(11)
C4 = Pin(12)
C5 = Pin(13)
C6 = Pin(14)
C7 = Pin(15)
# C8 and C9 are not GPIO

# create None type pins for I2C and SPI since they are expected to be defined
SCL = Pin()
SDA = Pin()
SCK = SCLK = Pin()
MOSI = Pin()
MISO = Pin()



"""Pin definitions for the STM32MP157C Development Kit 2."""

from adafruit_blinka.microcontroller.stm32.stm32mp157 import pin

D2 = pin.PA12
D3 = pin.PA11
D4 = pin.PA8
D5 = pin.PG2
D6 = pin.PH11
D7 = pin.PF3
D8 = pin.PF6
D9 = pin.PF8
D10 = pin.PF9
D11 = pin.PF7
D12 = pin.PD13
D13 = pin.PC7
D14 = pin.PB10
D15 = pin.PB12
D16 = pin.PB13
D17 = pin.PG8
D18 = pin.PI5
D19 = pin.PI7
D20 = pin.PI6
D21 = pin.PF11
D22 = pin.PG15
D23 = pin.PF1
D24 = pin.PF0
D25 = pin.PF4
D26 = pin.PF5
D27 = pin.PD7

SDA = D2
SCL = D3

SDA1 = pin.PF15
SCL1 = pin.PD12

SCLK = D11
MOSI = D10
MISO = D9

CE0 = D8
CE1 = D7

CS = CE0
SCK = SCLK

UART_TX = D14
UART_RX = D15



"""I2C Class for MCP2221"""
from .mcp2221 import mcp2221


class I2C:
    """Custom I2C Class for MCP2221"""

    def __init__(self, *, frequency=100000):
        self._mcp2221 = mcp2221
        self._mcp2221._i2c_configure(frequency)

    def scan(self):
        """Perform an I2C Device Scan"""
        return self._mcp2221.i2c_scan()

    # pylint: disable=unused-argument
    def writeto(self, address, buffer, *, start=0, end=None, stop=True):
        """Write data from the buffer to an address"""
        self._mcp2221.i2c_writeto(address, buffer, start=start, end=end)

    def readfrom_into(self, address, buffer, *, start=0, end=None, stop=True):
        """Read data from an address and into the buffer"""
        self._mcp2221.i2c_readfrom_into(address, buffer, start=start, end=end)

    def writeto_then_readfrom(
        self,
        address,
        buffer_out,
        buffer_in,
        *,
        out_start=0,
        out_end=None,
        in_start=0,
        in_end=None,
        stop=False
    ):
        """Write data from buffer_out to an address and then
        read data from an address and into buffer_in
        """
        self._mcp2221.i2c_writeto_then_readfrom(
            address,
            buffer_out,
            buffer_in,
            out_start=out_start,
            out_end=out_end,
            in_start=in_start,
            in_end=in_end,
        )

    # pylint: enable=unused-argument



"""
`bitbangio` - Bitbanged bus protocols
==============================================================

See `CircuitPython:bitbangio` in CircuitPython for more details.

* Author(s): cefn
"""

import adafruit_platformdetect.constants.boards as ap_board
from adafruit_blinka import Lockable, agnostic

# pylint: disable=import-outside-toplevel,too-many-arguments


class I2C(Lockable):
    """Bitbang/Software I2C implementation"""

    def __init__(self, scl, sda, frequency=400000):
        # TODO: This one is a bit questionable:
        if agnostic.board_id == ap_board.PYBOARD:
            raise NotImplementedError("No software I2C on {}".format(agnostic.board_id))
        if agnostic.detector.board.any_embedded_linux:
            # TODO: Attempt to load this library automatically
            raise NotImplementedError(
                "For bitbangio on Linux, please use Adafruit_CircuitPython_BitbangIO"
            )
        self.init(scl, sda, frequency)

    def init(self, scl, sda, frequency):
        """Initialization"""
        from machine import Pin
        from machine import I2C as _I2C

        self.deinit()
        id = (  # pylint: disable=redefined-builtin
            -1
        )  # force bitbanging implementation - in future
        # introspect platform if SDA/SCL matches hardware I2C
        self._i2c = _I2C(id, Pin(scl.id), Pin(sda.id), freq=frequency)

    def deinit(self):
        """Deinitialization"""
        try:
            del self._i2c
        except AttributeError:
            pass

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.deinit()

    def scan(self):
        """Scan for attached devices"""
        return self._i2c.scan()

    def readfrom_into(self, address, buffer, start=0, end=None):
        """Read from a device at specified address into a buffer"""
        if start != 0 or end is not None:
            if end is None:
                end = len(buffer)
            buffer = memoryview(buffer)[start:end]
        stop = True  # remove for efficiency later
        return self._i2c.readfrom_into(address, buffer, stop)

    def writeto(self, address, buffer, start=0, end=None, stop=True):
        """Write to a device at specified address from a buffer"""
        if start != 0 or end is not None:
            if end is None:
                return self._i2c.writeto(address, memoryview(buffer)[start:], stop)
            return self._i2c.writeto(address, memoryview(buffer)[start:end], stop)
        return self._i2c.writeto(address, buffer, stop)


# TODO untested, as actually busio.SPI was on
# tasklist https://github.com/adafruit/Adafruit_Micropython_Blinka/issues/2 :(
class SPI(Lockable):
    """Bitbang/Software SPI implementation"""

    def __init__(self, clock, MOSI=None, MISO=None):
        if agnostic.detector.board.any_embedded_linux:
            # TODO: Attempt to load this library automatically
            raise NotImplementedError(
                "For bitbangio on Linux, please use Adafruit_CircuitPython_BitbangIO"
            )
        from machine import SPI as _SPI

        self._spi = _SPI(-1)
        self._pins = (clock, MOSI, MISO)

    def configure(self, baudrate=100000, polarity=0, phase=0, bits=8):
        """Update the configuration"""
        from machine import Pin
        from machine import SPI as _SPI

        if self._locked:
            # TODO verify if _spi obj 'caches' sck, mosi, miso to
            # avoid storing in _attributeIds (duplicated in busio)
            # i.e. #init ignores MOSI=None rather than unsetting
            self._spi.init(
                baudrate=baudrate,
                polarity=polarity,
                phase=phase,
                bits=bits,
                firstbit=_SPI.MSB,
                sck=Pin(self._pins[0].id),
                mosi=Pin(self._pins[1].id),
                miso=Pin(self._pins[2].id),
            )
        else:
            raise RuntimeError("First call try_lock()")

    def write(self, buf):
        """Write to the SPI device"""
        return self._spi.write(buf)

    def readinto(self, buf):
        """Read from the SPI device into a buffer"""
        return self.readinto(buf)

    def write_readinto(self, buffer_out, buffer_in):
        """Write to the SPI device and read from the SPI device into a buffer"""
        return self.write_readinto(buffer_out, buffer_in)



"""
`neopixel_write` - NeoPixel precision timed writing support
===========================================================

See `CircuitPython:neopixel_write` in CircuitPython for more details.
Currently supported on Raspberry Pi only.

* Author(s): ladyada
"""

import sys

from adafruit_blinka.agnostic import detector

if detector.board.any_raspberry_pi:
    from adafruit_blinka.microcontroller.bcm283x import neopixel as _neopixel
elif "sphinx" in sys.modules:
    pass
else:
    raise NotImplementedError("Board not supported")


def neopixel_write(gpio, buf):
    """Write buf out on the given DigitalInOut."""
    return _neopixel.neopixel_write(gpio, buf)



"""Pin definitions for the Odroid C4."""

from adafruit_blinka.microcontroller.amlogic.s905x3 import pin

for it in pin.i2cPorts:
    globals()["SCL" + str(it[0])] = it[1]
    globals()["SDA" + str(it[0])] = it[2]

SCL = pin.i2cPorts[0][1]
SDA = pin.i2cPorts[0][2]

SCLK = pin.SPI0_SCLK
MOSI = pin.SPI0_MOSI
MISO = pin.SPI0_MISO
SPI_CS0 = pin.SPI0_CS0

"""J2: physical pin number"""

D3 = pin.GPIOX_17
D5 = pin.GPIOX_18
D7 = pin.GPIOX_5
D8 = pin.GPIOX_12
D10 = pin.GPIOX_13
D11 = pin.GPIOX_3
D12 = pin.GPIOX_16
D13 = pin.GPIOX_4
D15 = pin.GPIOX_7
D16 = pin.GPIOX_0
D18 = pin.GPIOX_1
D19 = pin.GPIOX_8
D21 = pin.GPIOX_9
D22 = pin.GPIOX_2
D23 = pin.GPIOX_11
D24 = pin.GPIOX_10
D26 = pin.GPIOH_6
D27 = pin.GPIOA_14
D28 = pin.GPIOA_15
D29 = pin.GPIOX_14
D31 = pin.GPIOX_15
D32 = pin.GPIOH_7
D33 = pin.GPIOX_6
D35 = pin.GPIOX_19
D36 = pin.GPIOH_5

"""J7: physical pin number"""

D42 = pin.GPIOAO_10
D44 = pin.GPIOAO_9
D45 = pin.GPIOAO_7
D46 = pin.GPIOAO_8
D47 = pin.GPIOAO_4



"""Tegra T186 pin names"""

import atexit
import Jetson.GPIO as GPIO

GPIO.setmode(GPIO.TEGRA_SOC)
GPIO.setwarnings(False)  # shh!


class Pin:
    """Pins dont exist in CPython so...lets make our own!"""

    IN = 0
    OUT = 1
    LOW = 0
    HIGH = 1
    PULL_NONE = 0
    PULL_UP = 1
    PULL_DOWN = 2

    id = None
    _value = LOW
    _mode = IN

    def __init__(self, bcm_number):
        self.id = bcm_number

    def __repr__(self):
        return str(self.id)

    def __eq__(self, other):
        return self.id == other

    def init(self, mode=IN, pull=None):
        """Initialize the Pin"""
        if mode is not None:
            if mode == self.IN:
                self._mode = self.IN
                GPIO.setup(self.id, GPIO.IN)
            elif mode == self.OUT:
                self._mode = self.OUT
                GPIO.setup(self.id, GPIO.OUT)
            else:
                raise RuntimeError("Invalid mode for pin: %s" % self.id)
        if pull is not None:
            if self._mode != self.IN:
                raise RuntimeError("Cannot set pull resistor on output")
            if pull == self.PULL_UP:
                GPIO.setup(self.id, GPIO.IN, pull_up_down=GPIO.PUD_UP)
            elif pull == self.PULL_DOWN:
                GPIO.setup(self.id, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)
            else:
                raise RuntimeError("Invalid pull for pin: %s" % self.id)

    def value(self, val=None):
        """Set or return the Pin Value"""
        if val is not None:
            if val == self.LOW:
                self._value = val
                GPIO.output(self.id, val)
                return None
            if val == self.HIGH:
                self._value = val
                GPIO.output(self.id, val)
                return None
            raise RuntimeError("Invalid value for pin")
        return GPIO.input(self.id)

    # pylint: disable=no-method-argument
    @atexit.register
    def cleanup():
        """Clean up pins"""
        print("Exiting... \nCleaning up pins")
        GPIO.cleanup()

    # pylint: enable=no-method-argument


# Cannot be used as GPIO
SDA = Pin("GPIO_SEN9")
SCL = Pin("GPIO_SEN8")
SDA_1 = Pin("GEN1_I2C_SDA")
SCL_1 = Pin("GEN1_I2C_SCL")

J04 = Pin("AUD_MCLK")
J06 = Pin("GPIO_AUD1")
AA02 = Pin("CAN_GPIO2")
N06 = Pin("GPIO_CAM7")
N04 = Pin("GPIO_CAM5")
N05 = Pin("GPIO_CAM6")
N03 = Pin("GPIO_CAM4")
AA01 = Pin("CAN_GPIO1")
I05 = Pin("GPIO_PQ5")
T03 = Pin("UART1_CTS")
T02 = Pin("UART1_RTS")
J00 = Pin("DAP1_SCLK")
J03 = Pin("DAP1_FS")
J02 = Pin("DAP1_DIN")
J01 = Pin("DAP1_DOUT")
P17 = Pin("GPIO_EXP_P17")
AA00 = Pin("CAN0_GPIO0")
Y01 = Pin("GPIO_MDM2")
P16 = Pin("GPIO_EXP_P16")
I04 = Pin("GPIO_PQ4")
J05 = Pin("GPIO_AUD0")

i2cPorts = (
    (1, SCL, SDA),
    (0, SCL_1, SDA_1),
)

# ordered as spiId, sckId, mosiId, misoId
spiPorts = ((3, N03, N05, N04),)



"""Pin definitions for the Tritium H3."""

from adafruit_blinka.microcontroller.allwinner.h3 import pin

PA12 = pin.PA12
SDA = pin.PA12
PA11 = pin.PA11
SCL = pin.PA11
PA2 = pin.PA2
PA1 = pin.PA1
PA0 = pin.PA0
PA3 = pin.PA3
PC0 = pin.PC0
MOSI = pin.PC0
PC1 = pin.PC1
MISO = pin.PC1
PC2 = pin.PC2
SCK = pin.PC2
SCLK = pin.PC2
PA19 = pin.PA19
PA20 = pin.PA20
PA21 = pin.PA21
PA6 = pin.PA6
PG10 = pin.PG10
PA16 = pin.PA16

PG6 = pin.PG6
TX = pin.PG6
PG7 = pin.PG7
RX = pin.PG7
PG11 = pin.PG11
PG9 = pin.PG9
PG8 = pin.PG8
PA14 = pin.PA14
PC3 = pin.PC3
PA17 = pin.PA17
PA18 = pin.PA18
PA13 = pin.PA13
PA15 = pin.PA15
PG13 = pin.PG13
PG12 = pin.PG12



"""Binho Nova pin names"""


class Pin:
    """A basic Pin class for use with Binho Nova."""

    IN = "DIN"
    OUT = "DOUT"
    AIN = "AIN"
    AOUT = "AOUT"
    PWM = "PWM"
    LOW = 0
    HIGH = 1

    _nova = None

    def __init__(self, pin_id=None):
        if not Pin._nova:
            # pylint: disable=import-outside-toplevel
            from adafruit_blinka.microcontroller.nova import Connection

            # pylint: enable=import-outside-toplevel

            Pin._nova = Connection.getInstance()
        # check if pin is valid
        if pin_id > 4:
            raise ValueError("Invalid pin {}.".format(pin_id))

        self.id = pin_id

    def init(self, mode=IN, pull=None):
        """Initialize the Pin"""
        if self.id is None:
            raise RuntimeError("Can not init a None type pin.")
        # Nova does't have configurable internal pulls for
        if pull:
            raise ValueError("Internal pull up/down not currently supported.")
        Pin._nova.setIOpinMode(self.id, mode)

    def value(self, val=None):
        """Set or return the Pin Value"""
        if self.id is None:
            raise RuntimeError("Can not access a None type pin.")
        # read
        if val is None:
            return int(Pin._nova.getIOpinValue(self.id).split("VALUE ")[1])
        # write
        if val in (self.LOW, self.HIGH):
            Pin._nova.setIOpinValue(self.id, val)
            return None
        raise RuntimeError("Invalid value for pin")


# create pin instances for each pin
IO0 = Pin(0)
IO1 = Pin(1)
IO2 = Pin(2)
IO3 = Pin(3)
IO4 = Pin(4)

SCL = IO2
SDA = IO0
SCK = SCLK = IO3
MOSI = IO4
MISO = IO2
SS0 = IO0
SS1 = IO1

PWM0 = IO0
# No PWM support on IO1
PWM2 = IO2
PWM3 = IO3
PWM4 = IO4

# orderd as (channel, pin), id
pwmOuts = (((1, 0), PWM0), ((1, 2), PWM2), ((1, 3), PWM3), ((1, 4), PWM4))

UART1_TX = IO4
UART1_RX = IO3

# ordered as uartId, txId, rxId
uartPorts = ((0, UART1_TX, UART1_RX),)



"""ESP8266 pin names"""

from microcontroller import Pin

GPIO0 = Pin(0)
GPIO1 = Pin(1)
GPIO2 = Pin(2)
GPIO3 = Pin(3)
GPIO4 = Pin(4)
GPIO5 = Pin(5)
GPIO12 = Pin(12)
GPIO13 = Pin(13)
GPIO14 = Pin(14)
GPIO15 = Pin(15)
GPIO16 = Pin(16)
TOUT = Pin("TOUT")

# ordered as spiId, sckId, mosiId, misoId
SPI_PORTS = (1, GPIO14, GPIO13, GPIO12)

# ordered as uartId, txId, rxId
UART_PORTS = (
    (0, GPIO1, GPIO3),
    # TODO secondary pins for UART0 configurable from Micropython? How to flag?
    # (0, GPIO15, GPIO13)
    (1, GPIO2, None),
)

I2C_PORTS = ()



"""A Pin class for use with periphery."""
try:
    from periphery import GPIO
except ImportError:
    raise ImportError(
        "Periphery Python bindings not found, please install and try again! "
        "Try running 'pip3 install python-periphery'"
    ) from ImportError


class Pin:
    """Pins dont exist in CPython so...lets make our own!"""

    IN = "in"
    OUT = "out"
    LOW = 0
    HIGH = 1
    PULL_NONE = 0
    PULL_UP = 1
    PULL_DOWN = 2

    id = None
    _value = LOW
    _mode = IN

    def __init__(self, pin_id):
        self.id = pin_id
        if isinstance(pin_id, tuple):
            self._num = int(pin_id[1])
            self._chippath = "/dev/gpiochip{}".format(pin_id[0])
        else:
            self._num = int(pin_id)
            self._chippath = "/dev/gpiochip0"
        self._line = None

    def __repr__(self):
        return str(self.id)

    def __eq__(self, other):
        return self.id == other

    def init(self, mode=IN, pull=None):
        """Initialize the Pin"""
        if mode is not None:
            if mode == self.IN:
                self._mode = self.IN
                if self._line is not None:
                    self._line.close()
                self._line = GPIO(self._chippath, int(self._num), self.IN)
            elif mode == self.OUT:
                self._mode = self.OUT
                if self._line is not None:
                    self._line.close()
                self._line = GPIO(self._chippath, int(self._num), self.OUT)
            else:
                raise RuntimeError("Invalid mode for pin: %s" % self.id)

            if pull is not None:
                if pull == self.PULL_UP:
                    raise NotImplementedError(
                        "Internal pullups not supported in periphery, "
                        "use physical resistor instead!"
                    )
                if pull == self.PULL_DOWN:
                    raise NotImplementedError(
                        "Internal pulldowns not supported in periphery, "
                        "use physical resistor instead!"
                    )
                raise RuntimeError("Invalid pull for pin: %s" % self.id)

    def value(self, val=None):
        """Set or return the Pin Value"""
        if val is not None:
            if val == self.LOW:
                self._value = val
                self._line.write(False)
                return None
            if val == self.HIGH:
                self._value = val
                self._line.write(True)
                return None
            raise RuntimeError("Invalid value for pin")
        return self.HIGH if self._line.read() else self.LOW



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Note: To use the 'upload' functionality of this file, you must:
#   $ pip install twine

import io
import os
import sys

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
with io.open(os.path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

board_reqs = []
if os.path.exists("/proc/device-tree/compatible"):
    with open("/proc/device-tree/compatible", "rb") as f:
        compat = f.read()
    if b"nvidia,tegra" in compat:
        board_reqs = ["Jetson.GPIO"]
    if (
        b"brcm,bcm2835" in compat
        or b"brcm,bcm2836" in compat
        or b"brcm,bcm2837" in compat
        or b"brcm,bcm2838" in compat
        or b"brcm,bcm2711" in compat
    ):
        board_reqs = ["RPi.GPIO", "rpi_ws281x>=4.0.0", "sysv_ipc>=1.1.0"]

setup(
    name="Adafruit-Blinka",
    use_scm_version=True,
    setup_requires=["setuptools_scm"],
    description="CircuitPython APIs for non-CircuitPython versions of Python such as CPython on Linux and MicroPython.",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    author="Adafruit Industries",
    author_email="circuitpython@adafruit.com",
    python_requires=">=3.6.0",
    url="https://github.com/adafruit/Adafruit_Blinka",
    package_dir={"": "src"},
    packages=find_packages("src"),
    # py_modules lists top-level single file packages to include.
    # find_packages only finds packages in directories with __init__.py files.
    py_modules=[
        "analogio",
        "bitbangio",
        "board",
        "busio",
        "digitalio",
        "micropython",
        "pulseio",
        "pwmio",
        "neopixel_write",
    ],
    package_data={
        "adafruit_blinka.microcontroller.bcm283x.pulseio": ["libgpiod_pulsein"]
    },
    install_requires=[
        "Adafruit-PlatformDetect>=3.1.0",
        "Adafruit-PureIO>=1.1.7",
        "pyftdi>=0.40.0",
    ]
    + board_reqs,
    license="MIT",
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: MicroPython",
    ],
)



"""Pin definitions for 40-pin Raspberry Pi models."""

from adafruit_blinka.microcontroller.bcm283x import pin

D0 = pin.D0
D1 = pin.D1

D2 = pin.D2
SDA = pin.SDA
D3 = pin.D3
SCL = pin.SCL

D4 = pin.D4
D5 = pin.D5
D6 = pin.D6

D7 = pin.D7
CE1 = pin.D7
D8 = pin.D8
CE0 = pin.D8
D9 = pin.D9
MISO = pin.D9
D10 = pin.D10
MOSI = pin.D10
D11 = pin.D11
SCLK = pin.D11
SCK = pin.D11

D12 = pin.D12
D13 = pin.D13

D14 = pin.D14
TXD = pin.D14
D15 = pin.D15
RXD = pin.D15
# create alias for most of the examples
TX = pin.D14
RX = pin.D15

D16 = pin.D16
D17 = pin.D17
D18 = pin.D18
D19 = pin.D19
MISO_1 = pin.D19
D20 = pin.D20
MOSI_1 = pin.D20
D21 = pin.D21
SCLK_1 = pin.D21
SCK_1 = pin.D21
D22 = pin.D22
D23 = pin.D23
D24 = pin.D24
D25 = pin.D25
D26 = pin.D26
D27 = pin.D27



"""Pin definitions for Jetson Nano."""

from adafruit_blinka.microcontroller.tegra.t210 import pin

SDA = pin.SDA_1
SCL = pin.SCL_1
SDA_1 = pin.SDA
SCL_1 = pin.SCL

D4 = pin.BB00
D5 = pin.S05
D6 = pin.Z00
D7 = pin.C04
D8 = pin.C03
D9 = pin.C01
D10 = pin.C00
D11 = pin.C02
D12 = pin.V00
D13 = pin.E06
D16 = pin.G03
D17 = pin.G02
D18 = pin.J07
D19 = pin.J04
D20 = pin.J05
D21 = pin.J06
D22 = pin.Y02
D23 = pin.DD00
D24 = pin.B07
D25 = pin.B05
D26 = pin.B04
D27 = pin.B06

CE1 = D7
CE0 = D8
MISO = D9
MOSI = D10
SCLK = D11
SCK = D11

CE1_1 = D23
CE0_1 = D24
MISO_1 = D25
MOSI_1 = D26
SCLK_1 = D27
SCK_1 = D27



from adafruit_blinka import agnostic

if agnostic.board in ("feather_m0_express", "feather_huzzah"):
    from bitbangio import I2C
elif agnostic.board == "pyboard":
    from busio import I2C
else:
    raise NotImplementedError("Board not supported")



# The MIT License (MIT)
#
# Copyright (c) 2017 cefn for adafruit industries
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""
`board` - Define ids for available pins
=================================================

See `CircuitPython:board` in CircuitPython for more details.

* Author(s): cefn
"""
import sys

import adafruit_platformdetect.constants.boards as ap_board
from adafruit_blinka.agnostic import board_id, detector

# pylint: disable=wildcard-import,unused-wildcard-import,ungrouped-imports
# pylint: disable=import-outside-toplevel

if board_id == ap_board.FEATHER_HUZZAH:
    from adafruit_blinka.board.feather_huzzah import *

elif board_id == ap_board.NODEMCU:
    from adafruit_blinka.board.nodemcu import *

elif board_id == ap_board.PYBOARD:
    from adafruit_blinka.board.pyboard import *

elif detector.board.any_raspberry_pi_40_pin:
    from adafruit_blinka.board.raspberrypi.raspi_40pin import *

elif detector.board.any_raspberry_pi_cm:
    from adafruit_blinka.board.raspberrypi.raspi_cm import *

elif detector.board.RASPBERRY_PI_B_REV1:
    from adafruit_blinka.board.raspberrypi.raspi_1b_rev1 import *

elif detector.board.RASPBERRY_PI_A or detector.board.RASPBERRY_PI_B_REV2:
    from adafruit_blinka.board.raspberrypi.raspi_1b_rev2 import *

elif board_id == ap_board.BEAGLEBONE_BLACK:
    from adafruit_blinka.board.beagleboard.beaglebone_black import *

elif board_id == ap_board.BEAGLEBONE_GREEN:
    from adafruit_blinka.board.beagleboard.beaglebone_black import *

elif board_id == ap_board.BEAGLEBONE_BLACK_INDUSTRIAL:
    from adafruit_blinka.board.beagleboard.beaglebone_black import *

elif board_id == ap_board.BEAGLEBONE_GREEN_WIRELESS:
    from adafruit_blinka.board.beagleboard.beaglebone_black import *

elif board_id == ap_board.BEAGLEBONE_BLACK_WIRELESS:
    from adafruit_blinka.board.beagleboard.beaglebone_black import *

elif board_id == ap_board.BEAGLEBONE_POCKETBEAGLE:
    from adafruit_blinka.board.beagleboard.beaglebone_pocketbeagle import *

elif board_id == ap_board.BEAGLEBONE_AI:
    from adafruit_blinka.board.beagleboard.beaglebone_ai import *

elif board_id == ap_board.ORANGE_PI_PC:
    from adafruit_blinka.board.orangepi.orangepipc import *

elif board_id == ap_board.ORANGE_PI_R1:
    from adafruit_blinka.board.orangepi.orangepir1 import *

elif board_id == ap_board.ORANGE_PI_ZERO:
    from adafruit_blinka.board.orangepi.orangepizero import *

elif board_id == ap_board.ORANGE_PI_ONE:
    from adafruit_blinka.board.orangepi.orangepipc import *

elif board_id == ap_board.ORANGE_PI_PC_PLUS:
    from adafruit_blinka.board.orangepi.orangepipc import *

elif board_id == ap_board.ORANGE_PI_LITE:
    from adafruit_blinka.board.orangepi.orangepipc import *

elif board_id == ap_board.ORANGE_PI_PLUS_2E:
    from adafruit_blinka.board.orangepi.orangepipc import *

elif board_id == ap_board.ORANGE_PI_2:
    from adafruit_blinka.board.orangepi.orangepipc import *

elif board_id == ap_board.ORANGE_PI_ZERO_PLUS_2H5:
    from adafruit_blinka.board.orangepi.orangepizeroplus2h5 import *

elif board_id == ap_board.ORANGE_PI_ZERO_PLUS:
    from adafruit_blinka.board.orangepi.orangepizeroplus import *

elif board_id == ap_board.ORANGE_PI_ZERO_2:
    from adafruit_blinka.board.orangepi.orangepizero2 import *

elif board_id == ap_board.BANANA_PI_M2_ZERO:
    from adafruit_blinka.board.bananapi.bpim2zero import *

elif board_id == ap_board.GIANT_BOARD:
    from adafruit_blinka.board.giantboard import *

elif board_id == ap_board.JETSON_TX1:
    from adafruit_blinka.board.nvidia.jetson_tx1 import *

elif board_id == ap_board.JETSON_TX2:
    from adafruit_blinka.board.nvidia.jetson_tx2 import *

elif board_id == ap_board.JETSON_XAVIER:
    from adafruit_blinka.board.nvidia.jetson_xavier import *

elif board_id == ap_board.JETSON_NANO:
    from adafruit_blinka.board.nvidia.jetson_nano import *

elif board_id == ap_board.JETSON_NX:
    from adafruit_blinka.board.nvidia.jetson_nx import *

elif board_id == ap_board.CLARA_AGX_XAVIER:
    from adafruit_blinka.board.nvidia.clara_agx_xavier import *

elif board_id == ap_board.CORAL_EDGE_TPU_DEV:
    from adafruit_blinka.board.coral_dev_board import *

elif board_id == ap_board.CORAL_EDGE_TPU_DEV_MINI:
    from adafruit_blinka.board.coral_dev_board_mini import *

elif board_id == ap_board.ODROID_C2:
    from adafruit_blinka.board.hardkernel.odroidc2 import *

elif board_id == ap_board.ODROID_C4:
    from adafruit_blinka.board.hardkernel.odroidc4 import *

elif board_id == ap_board.ODROID_N2:
    from adafruit_blinka.board.hardkernel.odroidn2 import *

elif board_id == ap_board.ODROID_XU4:
    from adafruit_blinka.board.hardkernel.odroidxu4 import *

elif board_id == ap_board.DRAGONBOARD_410C:
    from adafruit_blinka.board.dragonboard_410c import *

elif board_id == ap_board.FTDI_FT232H:
    from adafruit_blinka.board.ftdi_ft232h import *

elif board_id == ap_board.BINHO_NOVA:
    from adafruit_blinka.board.binho_nova import *

elif board_id == ap_board.MICROCHIP_MCP2221:
    from adafruit_blinka.board.microchip_mcp2221 import *

elif board_id == ap_board.GREATFET_ONE:
    from adafruit_blinka.board.greatfet_one import *

elif board_id == ap_board.SIFIVE_UNLEASHED:
    from adafruit_blinka.board.hifive_unleashed import *

elif board_id == ap_board.PINE64:
    from adafruit_blinka.board.pine64 import *

elif board_id == ap_board.PINEH64:
    from adafruit_blinka.board.pineH64 import *

elif board_id == ap_board.SOPINE:
    from adafruit_blinka.board.soPine import *

elif board_id == ap_board.CLOCKWORK_CPI3:
    from adafruit_blinka.board.clockworkcpi3 import *

elif board_id == ap_board.ONION_OMEGA2:
    from adafruit_blinka.board.onion.omega2 import *

elif board_id == ap_board.ROCK_PI_S:
    from adafruit_blinka.board.radxa.rockpis import *

elif board_id == ap_board.UDOO_X86:
    from adafruit_blinka.board.udoo_x86ultra import *

elif board_id == ap_board.STM32MP157C_DK2:
    from adafruit_blinka.board.stm32.stm32mp157c_dk2 import *

elif board_id == ap_board.LUBANCAT_IMX6ULL:
    from adafruit_blinka.board.lubancat.lubancat_imx6ull import *

elif board_id == ap_board.NANOPI_NEO_AIR:
    from adafruit_blinka.board.nanopi.neoair import *

elif board_id == ap_board.NANOPI_DUO2:
    from adafruit_blinka.board.nanopi.duo2 import *

elif "sphinx" in sys.modules:
    pass

else:
    raise NotImplementedError("Board not supported {}".format(board_id))


def I2C():
    """The singleton I2C interface"""
    import busio

    return busio.I2C(SCL, SDA)


def SPI():
    """The singleton SPI interface"""
    import busio

    return busio.SPI(SCLK, MOSI, MISO)



import time
import board
import busio

spi = busio.SPI(board.SCLK, board.MOSI, board.MISO)
while not spi.try_lock():
    pass
spi.configure(baudrate=16000000)
spi.unlock()

while True:
    spi.write(bytes(range(64)))
    time.sleep(0.1)



import unittest
from testing import yes_no, await_true
from testing.board import led_pin, default_pin, led_hardwired, led_inverted
from digitalio import *


class TestDigitalInOut(unittest.TestCase):
    def test_default(self):
        """DigitalInOut is input with no pull when constructed"""
        with DigitalInOut(default_pin) as dio:
            self.assertEqual(dio.direction, Direction.INPUT)
            self.assertEqual(dio.pull, None)

    def test_switch_to_output(self):
        """Default configuration of switch_to_output is respected"""
        with DigitalInOut(default_pin) as dio:
            dio.switch_to_output()
            self.assertEqual(dio.direction, Direction.OUTPUT)
            self.assertEqual(dio.value, False)
            self.assertEqual(dio.drive_mode, DriveMode.PUSH_PULL)

    def test_switch_to_input(self):
        """Default configuration of switch_to_input is respected"""
        with DigitalInOut(default_pin) as dio:
            dio.switch_to_output()  # starts as input anyway
            dio.switch_to_input()
            self.assertEqual(dio.direction, Direction.INPUT)
            self.assertEqual(dio.pull, None)


class TestDigitalInOutInteractive(unittest.TestCase):
    def test_blink(self):
        """LED blinks when proper attributes set"""
        print()
        from adafruit_blinka.agnostic import sleep

        if not (led_hardwired) and not (yes_no("Is LED wired to {}".format(led_pin))):
            return  # test trivially passed
        with DigitalInOut(led_pin) as led:
            led.direction = Direction.OUTPUT
            # should now be OUT, PUSH_PULL, value=0, and LED should light
            led.value = False if led_inverted else True
            self.assertTrue(yes_no("Is LED lit"))
            print("Winking LED...")
            for count in range(2):
                led.value = not (led.value)
                sleep(0.5)
                led.value = not (led.value)
                sleep(0.5)
            self.assertTrue(yes_no("Did LED wink twice"))

    def test_button_pull_up(self):
        print()
        """Pull-up button configured and detected"""
        with DigitalInOut(default_pin) as button:
            # button.direction = Direction.INPUT # implied
            try:
                button.pull = Pull.UP
            except NotImplementedError as e:
                print(e)
                return  # pull unsupported, test trivially passed
            except Exception as e:
                print(e)
                return  # pull unsupported, test trivially passed
            if yes_no("Is Button wired from {} to GND".format(default_pin)):
                self.assertTrue(button.value == True)
                self.assertTrue(
                    await_true("button pressed", lambda: button.value == False)
                )

    def test_button_pull_down(self):
        print()
        """Pull-down button configured and detected"""
        with DigitalInOut(default_pin) as button:
            # button.direction = Direction.INPUT # implied
            try:
                button.pull = Pull.DOWN
            except NotImplementedError as e:
                print(e)
                return  # pull unsupported, test trivially passed
            except Exception as e:
                print(e)
                return  # pull unsupported, test trivially passed
            if yes_no("Is Button wired from {} to VCC".format(default_pin)):
                self.assertTrue(button.value == False)
                self.assertTrue(
                    await_true("button pressed", lambda: button.value == True)
                )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Example of blinking LED on BeagleBone Black
# https://www.adafruit.com/product/1876
#
# Wire the circuit as follows:
# 1) connect anode (+) lead of LED to P9.12 pin
# 2) connect cathode (-) lead to 1K Ohm resistor
# 3) connect that 1K Ohm resistor to DGND (P9.1)
#
# NOTE: the pin mode can be verified with the command line
# utility config-pin on the BeagleBoard.org Debian image
#
# To verify the pin is in GPIO mode:
# debian@beaglebone:~$ config-pin -q p9.12
# P9_12 Mode: gpio Direction: out Value: 0
#
# To set pin to GPIO mode:
# $ config-pin p9.12 gpio

import time
import board
import digitalio

print("hello blinky!")

led = digitalio.DigitalInOut(board.P9_12)
led.direction = digitalio.Direction.OUTPUT

while True:
    led.value = True
    time.sleep(0.5)
    led.value = False
    time.sleep(0.5)



"""Pin definitions for Jetson TX1."""

from adafruit_blinka.microcontroller.tegra.t210 import pin

SDA = pin.SDA
SCL = pin.SCL
SDA_1 = pin.SDA_1
SCL_1 = pin.SCL_1

D4 = pin.BB00
D5 = pin.BB03
D6 = pin.X02
D7 = pin.C04
D8 = pin.C03
D9 = pin.C01
D10 = pin.C00
D11 = pin.C02
D12 = pin.E04
D13 = pin.H07
D16 = pin.U03
D17 = pin.U02
D18 = pin.B03
D19 = pin.B00
D20 = pin.B01
D21 = pin.B02
D22 = pin.P17
D23 = pin.E05
D24 = pin.X00
D25 = pin.P16
D26 = pin.X03
D27 = pin.E06

CE1 = D7
CE0 = D8
MISO = D9
MOSI = D10
SCLK = D11
SCK = D11



"""Pin definitions for the Coral Dev Board."""

from adafruit_blinka.microcontroller.nxp_imx8m import pin

# Board name = RPI name [= alias] = pin name
I2C2_SDA = D2 = SDA = pin.I2C2_SDA
I2C2_SCL = D3 = SCL = pin.I2C2_SCL

PWM1 = D12 = pin.PWM1
PWM2 = D13 = pin.PWM2
PWM3 = D22 = pin.PWM3

GPIO_P13 = D27 = pin.GPIO6
GPIO_P16 = D23 = pin.GPIO73
GPIO_P18 = D24 = pin.GPIO138
GPIO_P29 = D5 = pin.GPIO7
GPIO_P31 = D6 = pin.GPIO8
GPIO_P36 = D16 = pin.GPIO141
GPIO_P37 = D26 = pin.GPIO77

ECSPI1_MISO = D9 = MISO = pin.ECSPI1_MISO
ECSPI1_MOSI = D10 = MOSI = pin.ECSPI1_MOSI
ECSPI1_SCLK = D11 = SCLK = SCK = pin.ECSPI1_SCLK
ECSPI1_SS0 = D8 = SS0 = pin.ECSPI1_SS0



"""Pin definitions for the Odroid XU4."""

from adafruit_blinka.microcontroller.samsung.exynos5422 import pin

SDA = SDA1 = pin.I2C1_SDA
SCL = SCL1 = pin.I2C1_SCL
SDA5 = pin.I2C5_SDA
SCL5 = pin.I2C5_SCL

SCLK = pin.SPI1_SCLK
MOSI = pin.SPI1_MOSI
MISO = pin.SPI1_MISO
SPI_CS0 = pin.SPI1_CS0

"""physical pin number(XU4 Shifter Shield)"""

D3 = pin.GPIOB3_2
D5 = pin.GPIOB3_3
D7 = pin.GPIOX1_2
D8 = pin.GPIOA0_1
D10 = pin.GPIOA0_0
D11 = pin.GPIOA0_3
D12 = pin.GPIOA0_2
D13 = pin.GPIOX1_5
D15 = pin.GPIOX1_6
D16 = pin.GPIOX1_3
D18 = pin.GPIOX1_7
D19 = pin.GPIOA2_7
D21 = pin.GPIOA2_6
D22 = pin.GPIOX2_0
D23 = pin.GPIOA2_4
D24 = pin.GPIOA2_5
D26 = pin.GPIOX2_1
D27 = pin.GPIOA2_2
D28 = pin.GPIOA2_4
D29 = pin.GPIOX2_4
D31 = pin.GPIOX2_6
D32 = pin.GPIOX2_5
D33 = pin.GPIOX2_7
D36 = pin.GPIOX3_1



"""Pin definitions for the Banana Pi M2 Zero."""

# The Banana Pi M2 Zero uses the AllWinner H2 SoC, but pins
# are the same as the AllWinner H3 SoC, so we import those
from adafruit_blinka.microcontroller.allwinner.h3 import pin

PA12 = pin.PA12
SDA = pin.PA12
PA11 = pin.PA11
SCL = pin.PA11
PA6 = pin.PA6
PWM1 = pin.PA6
PA1 = pin.PA1
UART2_RX = pin.PA1
PA0 = pin.PA0
UART2_TX = pin.PA0
PA3 = pin.PA3
UART2_CTS = pin.PA3
PA10 = pin.PA10
PA12 = pin.PA12
SDA = pin.PA12
PA11 = pin.PA11
SCL = pin.PA11
PA6 = pin.PA6
PWM1 = pin.PA6
PA1 = pin.PA1
UART2_RX = pin.PA1
PA0 = pin.PA0
UART2_TX = pin.PA0
PA3 = pin.PA3
UART2_CTS = pin.PA3
PA7 = pin.PA7
PA8 = pin.PA8
PA9 = pin.PA9
PA10 = pin.PA10
PA17 = pin.PA17
PA18 = pin.PA18
PA19 = pin.PA19
PA20 = pin.PA20
PA21 = pin.PA21
PC0 = pin.PC0
PC1 = pin.PC1
PC2 = pin.PC2
PC3 = pin.PC3
PC4 = pin.PC4
PC7 = pin.PC7

PA13 = pin.PA13
SPI1_CS = pin.PA13
PA14 = pin.PA14
SPI1_CLK = pin.PA14
PA2 = pin.PA2
UART2_RTS = pin.PA2
PA18 = pin.PA18
TWI1_SCK = pin.PA18

PL2 = pin.PL2
PL4 = pin.PL4

SCLK = pin.PA14
MOSI = pin.PA15
MISO = pin.PA16

PA13 = pin.PA13
SPI1_CS = pin.PA13
PA14 = pin.PA14
SPI1_CLK = pin.PA14
PA2 = pin.PA2
UART2_RTS = pin.PA2
PA18 = pin.PA18
TWI1_SCK = pin.PA18
PG6 = pin.PG6
UART1_TX = pin.PG6
PG7 = pin.PG7
UART1_RX = pin.PG7

SCLK = pin.PA14
MOSI = pin.PA15
MISO = pin.PA16



import unittest


class TestMicrocontrollerModule(unittest.TestCase):
    def test_pins_exist(self):
        """The microcontroller module should contain pin references"""
        import microcontroller
        from microcontroller import pin
        from testing.microcontroller import pin_count

        entries = [getattr(pin, key) for key in dir(pin)]
        # is this filter line needed? any other types valid in pin module?
        entries = list(filter(lambda val: type(val) is microcontroller.Pin, entries))
        self.assertTrue(len(entries) > 0)
        self.assertTrue(len(entries) == pin_count)
